/* Copyright (c) 2010 - 2015 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.

   Portions Copyright (c) 2015-2017 Analog Devices, Inc.
   ---------------------------------------------------------------------------*/
/***********************************************************************/
/*                                                                     */
/*  FlashDev.C:  Flash Programming Functions adapted                   */
/*               for ADuCM302x 256kB Flash                             */
/*                                                                     */
/***********************************************************************/

#include "FlashOS.H"        // FlashOS Structures

/*
   Mandatory Flash Programming Functions (Called by FlashOS):
                int Init        (unsigned long adr,   // Initialize Flash
                                 unsigned long clk,
                                 unsigned long fnc);
                int UnInit      (unsigned long fnc);  // De-initialize Flash
                int EraseSector (unsigned long adr);  // Erase Sector Function
                int ProgramPage (unsigned long adr,   // Program Page Function
                                 unsigned long sz,
                                 unsigned char *buf);

   Optional  Flash Programming Functions (Called by FlashOS):
                int BlankCheck  (unsigned long adr,   // Blank Check
                                 unsigned long sz,
                                 unsigned char pat);
                int EraseChip   (void);               // Erase complete Device
      unsigned long Verify      (unsigned long adr,   // Verify Function
                                 unsigned long sz,
                                 unsigned char *buf);

       - BlanckCheck  is necessary if Flash space is not mapped into CPU memory space
       - Verify       is necessary if Flash space is not mapped into CPU memory space
       - if EraseChip is not provided than EraseSector for all sectors is called
*/

/* local types */
typedef unsigned char       uint8_t;
typedef unsigned short     uint16_t;
typedef unsigned long      uint32_t;
typedef unsigned long long uint64_t;

#define RESULT_OK                   0
#define RESULT_ERROR                1


/* Flash User Key */
#define FEE_USERKEY                        (0x676C7565)  /* (ascii "GLUE", in hex) */

/* Flash Commands */
#define FEE_CMD_IDLE                       (0x0)
#define FEE_CMD_ABORT                      (0x1)
#define FEE_CMD_SLEEP                      (0x2)
#define FEE_CMD_SIGN                       (0x3)
#define FEE_CMD_WRITE                      (0x4)
#define FEE_CMD_CHECK                      (0x5)
#define FEE_CMD_ERASEPAGE                  (0x6)
#define FEE_CMD_MASSERASE                  (0x7)

/* Flash Status */
#define FEE_STA_CMD_BUSY                   (0x1 << 0)
#define FEE_STA_WRITE_CLOSED               (0x1 << 1)
#define FEE_STA_CMD_COMPLETE               (0x1 << 2)
#define FEE_STA_WRITE_ALMOST_COMPLETE      (0x1 << 3)

#define FEE_STA_CMD_RESULT_SUCCESS         (0 << 4)
#define FEE_STA_CMD_RESULT_IGNORED         (1 << 4)
#define FEE_STA_CMD_RESULT_VERIFY_ERROR    (2 << 4)
#define FEE_STA_CMD_RESULT_ABORTED         (3 << 4)
#define FEE_STA_CMD_RESULT_MASK            ( FEE_STA_CMD_RESULT_SUCCESS      \
                                           | FEE_STA_CMD_RESULT_IGNORED      \
                                           | FEE_STA_CMD_RESULT_VERIFY_ERROR \
                                           | FEE_STA_CMD_RESULT_ABORTED )



/* the following macros (from ADuCM302x.h and ADuCM302x_cdef.h) are used locally... */

/*  WDT0 Control */
#define REG_WDT0_CTL                         0x40002C08
#define pREG_WDT0_CTL                    ((volatile  uint16_t *) REG_WDT0_CTL)

#define AFEWDT_WDTCON                        0x400c0908
#define pAFEWDT_WDTCON                   ((volatile  uint16_t *) AFEWDT_WDTCON)


/*  NVIC0 IRQ0..31 Clear_Enable */
#define REG_NVIC0_INTCLRE0                   0xE000E180
#define pREG_NVIC0_INTCLRE0              ((volatile  uint32_t *) REG_NVIC0_INTCLRE0)

/*  NVIC0 IRQ32..63 Clear_Enable */
#define REG_NVIC0_INTCLRE1                   0xE000E184
#define pREG_NVIC0_INTCLRE1              ((volatile  uint32_t *) REG_NVIC0_INTCLRE1)

/*  NVIC0 Systick Control and Status */
#define REG_NVIC0_STKSTA                     0xE000E010
#define pREG_NVIC0_STKSTA                ((volatile  uint32_t *) REG_NVIC0_STKSTA)

/*  NVIC0 Configurable Fault Status */
#define REG_NVIC0_INTCFSR                    0xE000ED28
#define pREG_NVIC0_INTCFSR               ((volatile  uint32_t *) REG_NVIC0_INTCFSR)

/*  NVIC0 Configurable Fault Status */
#define REG_NVIC0_INTCFSR                    0xE000ED28
#define pREG_NVIC0_INTCFSR               ((volatile  uint32_t *) REG_NVIC0_INTCFSR)

/*  CLKG0_OSC Key Protection for OSCCTRL */
#define REG_CLKG0_OSC_KEY                    0x4004C10C
#define pREG_CLKG0_OSC_KEY               ((volatile  uint32_t *) REG_CLKG0_OSC_KEY)

/*  CLKG0_OSC Oscillator Control */
#define REG_CLKG0_OSC_CTL                    0x4004C110
#define pREG_CLKG0_OSC_CTL               ((volatile  uint32_t *) REG_CLKG0_OSC_CTL)

/*  CLKG0_CLK Misc clock settings */
#define REG_CLKG0_CLK_CTL0                   0x4004C300
#define pREG_CLKG0_CLK_CTL0              ((volatile  uint32_t *) REG_CLKG0_CLK_CTL0)

/*  FLCC0 Interrupt Enable */
#define REG_FLCC0_IEN                        0x40018004
#define pREG_FLCC0_IEN                   ((volatile  uint32_t *) REG_FLCC0_IEN)

/*  FLCC0 Status */
#define REG_FLCC0_STAT                       0x40018000
#define pREG_FLCC0_STAT                  ((volatile  uint32_t *) REG_FLCC0_STAT)

/*  FLCC0 Key */
#define REG_FLCC0_KEY                        0x40018020
#define pREG_FLCC0_KEY                   ( (volatile  uint32_t *) REG_FLCC0_KEY)

/*  FLCC0 Command */
#define REG_FLCC0_CMD                        0x40018008
#define pREG_FLCC0_CMD                   ((volatile  uint32_t *) REG_FLCC0_CMD)

/*  FLCC0 Status */
#define REG_FLCC0_STAT                       0x40018000
#define pREG_FLCC0_STAT                  ((volatile  uint32_t *) REG_FLCC0_STAT)

/*  FLCC0 Lower page address */
#define REG_FLCC0_PAGE_ADDR0                 0x40018018
#define pREG_FLCC0_PAGE_ADDR0            ((volatile  uint32_t *) REG_FLCC0_PAGE_ADDR0)

/*  FLCC0 WRITE lower data */
#define REG_FLCC0_KH_DATA0                   0x40018010
#define pREG_FLCC0_KH_DATA0              ( (volatile  uint32_t *) REG_FLCC0_KH_DATA0)

/*  FLCC0 WRITE upper data */
#define REG_FLCC0_KH_DATA1                   0x40018014
#define pREG_FLCC0_KH_DATA1              ( (volatile  uint32_t *) REG_FLCC0_KH_DATA1)

/*  FLCC0 WRITE address */
#define REG_FLCC0_KH_ADDR                    0x4001800C
#define pREG_FLCC0_KH_ADDR               ((volatile  uint32_t *) REG_FLCC0_KH_ADDR)

/*  FLCC0 ECC_CFG address */
#define REG_FLCC0_ECC_CFG                    0x40018044
#define pREG_FLCC0_ECC_CFG               ((volatile  uint32_t *) REG_FLCC0_ECC_CFG)

/* FLCC0 ECC_CFG bits */
#define BITM_FLCC_ECC_CFG_INFOEN         0x00000002UL    /*  Info Space ECC Enable Bit */
#define BITM_FLCC_ECC_CFG_EN             0x00000001UL    /*  ECC Enable */


/* Saved ECC configuration */
static uint32_t saved_ecc_cfg = 0;


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

// NOTE: clk & fnc are unused here...

	uint8_t* pPageZeroMapBit;
	
	// size check
	if ( (sizeof(uint8_t)  != 1) || (sizeof(uint16_t) != 2) || (sizeof(uint32_t) != 4) || (sizeof(uint64_t) != 8) )
		return RESULT_ERROR;
	
	// Try to turn off the watchdog while we are programming flash
    *pREG_WDT0_CTL = 0;
    
    *pAFEWDT_WDTCON &=0xDF;

    // Disable all NVIC IRQs
    *pREG_NVIC0_INTCLRE0 = 0xFFFFFFFF;
    *pREG_NVIC0_INTCLRE1 = 0xFFFFFFFF;

    // SysTick control register "reset"
    *pREG_NVIC0_STKSTA   = 0x0;

    //NVIC0_INTVEC      = (unsigned long)VectorTable;  /* no IVT contained herein */
    *pREG_NVIC0_INTCFSR  = 0xFFFFFFFF; // Clear all bits that are currently set

    /* Enable the usage fault, bus fault and mem-manage fault handlers*/
    *pREG_NVIC0_INTCFSR = 0x00070000;

    // ensure user/info space page0 mapping bit is clear
    // so user space page0 is active page0 for flashing
    pPageZeroMapBit = (uint8_t*)0x40018054;  // reserved address
    *pPageZeroMapBit = 1;  // (W1C)

    // ensure internal HF oscillator is enabled
    // so flash has a guaranteed clock
    *pREG_CLKG0_OSC_KEY  = 0xcb14;  // unlock key
    *pREG_CLKG0_OSC_CTL |= 2;        // enable bit

    // ensure rclk mux select is set for internal HF oscillator
    *pREG_CLKG0_CLK_CTL0 &= ~(0x0300);  // set rclk mux select to zero

    // Disable any flash related interrupts
    *pREG_FLCC0_IEN = 0;

    // Save ECC configuration
    saved_ecc_cfg = *pREG_FLCC0_ECC_CFG;

    // Enable ECC for both user and info flash, and set page pointer to zero
    *pREG_FLCC0_ECC_CFG = BITM_FLCC_ECC_CFG_EN | BITM_FLCC_ECC_CFG_INFOEN;

    return RESULT_OK;
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

    // NOTE: fnc is unused here...

    // Restore saved ECC configuration
    *pREG_FLCC0_ECC_CFG = saved_ecc_cfg;

    // Clear the user key
    *pREG_FLCC0_KEY = 0;

    return RESULT_OK;                                  // Finished without Errors
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {

    uint32_t Result = RESULT_OK;
    volatile uint32_t status;

    // Clear any status from any previous command
    *pREG_FLCC0_STAT = 0xffffffff; /* W1C */

    // issue the mass erase command
    *pREG_FLCC0_KEY  = FEE_USERKEY;
    *pREG_FLCC0_CMD  = FEE_CMD_MASSERASE;

    // Wait for the command to complete
    do {
        status = *pREG_FLCC0_STAT;
    } while ((status & FEE_STA_CMD_COMPLETE) == 0x0);

    // Check result of operation
    if ((status & FEE_STA_CMD_RESULT_MASK) != FEE_STA_CMD_RESULT_SUCCESS) {
        Result = RESULT_ERROR; // Comand failed!
    }

    /* Clear key */
    *pREG_FLCC0_KEY = 0;

    return Result;
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {

    uint32_t Result = RESULT_OK;
    volatile uint32_t status;

    // Clear any status from any previous command
    *pREG_FLCC0_STAT = 0xffffffff; /* W1C */

    // Write the page start address
    *pREG_FLCC0_PAGE_ADDR0 = ((uint32_t)adr);

    // page erase command has implicit extent;  no need to set end address!

    // Issue the page erase command
    *pREG_FLCC0_KEY = FEE_USERKEY;
    *pREG_FLCC0_CMD = FEE_CMD_ERASEPAGE;

    // Wait for the command to complete
    do {
        status = *pREG_FLCC0_STAT;
    } while ((status & FEE_STA_CMD_COMPLETE) == 0x0);

    // Check result of operation
    if ((status & FEE_STA_CMD_RESULT_MASK) != FEE_STA_CMD_RESULT_SUCCESS) {
        Result = RESULT_ERROR; // Comand failed!
    }

    /* Clear key */
    *pREG_FLCC0_KEY = 0;

    return Result;
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {

    uint32_t Result = RESULT_OK;
    volatile uint32_t status;

    /* for ADuCM302x: all flash writes are 64-bit wide with INDIRECT addressing via Key-Hole registers... */

    /* initialize source and destination pointers as 64-bit types */
    uint64_t *pDst = (uint64_t *)((uint64_t *)adr);
    uint64_t *pSrc = (uint64_t *)(buf);

    // Write the user key
    *pREG_FLCC0_KEY = FEE_USERKEY;

    // iterate over the count
    for ( ; sz; sz -= sizeof(uint64_t), pSrc++, pDst++) {

        // clear status
        *pREG_FLCC0_STAT = 0xffffffff; /* W1C */

        if (sz >= sizeof(uint64_t)) {

            /* full 64-bit writes... */
            *pREG_FLCC0_KH_DATA0 = (uint32_t) (*pSrc);
            *pREG_FLCC0_KH_DATA1 = (uint32_t) (*pSrc >> 32);

        } else {

            /* partial 64-bit write, padded with all-ones */
            uint64_t u64TmpData  = 0xFFFFFFFFFFFFFFFF;
            uint8_t *pTmp = (uint8_t *)&u64TmpData;

            switch (sz) {

                // fallthrough writes all lower bytes
                case 7: pTmp[6] = *(((uint8_t *)pSrc)+ 6);
                case 6: pTmp[5] = *(((uint8_t *)pSrc)+ 5);
                case 5: pTmp[4] = *(((uint8_t *)pSrc)+ 4);
                case 4: pTmp[3] = *(((uint8_t *)pSrc)+ 3);
                case 3: pTmp[2] = *(((uint8_t *)pSrc)+ 2);
                case 2: pTmp[1] = *(((uint8_t *)pSrc)+ 1);
                case 1: pTmp[0] = *(((uint8_t *)pSrc)+ 0);
                    break;

                default:
                    break;
            }

            /* copy the 64-bit data to keyhole write data registers */
            *pREG_FLCC0_KH_DATA0 = (uint32_t) (u64TmpData);
            *pREG_FLCC0_KH_DATA1 = (uint32_t) (u64TmpData >> 32);

            sz = sizeof(uint64_t);  // forces loop termination after this iteration

        }  // end if/else block

        /* set the write address and issue the write */
        *pREG_FLCC0_KH_ADDR = (unsigned long)pDst;
        *pREG_FLCC0_CMD     = FEE_CMD_WRITE;

        /* verify write command was accepted without error */
        if (*pREG_FLCC0_STAT & FEE_STA_CMD_RESULT_MASK) {
            Result = RESULT_ERROR; // Comand failed!
            break;
        }

        // poll for command complete
        do {
            status = *pREG_FLCC0_STAT;
        } while ((status & FEE_STA_CMD_COMPLETE) == 0x0);

    }  // end for loop

    // Clear the user key
    *pREG_FLCC0_KEY = 0;

    return Result;
}
